#define MYSTDIOC
#include <mystdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <strings.h>

MYFILE	*myfopen(const char *filename,const char *mode) {
  MYFILE *stream;
  int real_mode;
  int myfile_mode;

  if (mode==NULL || strlen(mode)!=1) return NULL;
  real_mode = 0;
  myfile_mode = 0;
  switch(*mode) {
  case 'r':
    real_mode |= O_RDONLY;
    myfile_mode |= OPEN_READ;
    break;
  case 'w':
    real_mode |= O_WRONLY;
    real_mode |= O_CREAT;
    real_mode |= O_TRUNC;
    myfile_mode |= OPEN_WRITE;
    break;
  case 'a':
    real_mode |= O_WRONLY;
    real_mode |= O_APPEND;
    real_mode |= O_CREAT;
    myfile_mode |= OPEN_APPEND;
    break;
  case 'x':
    real_mode |= O_RDWR;
    real_mode |= O_CREAT;
    myfile_mode |= OPEN_READ;
    myfile_mode |= OPEN_WRITE;
    break;
  }

  stream = (MYFILE *)malloc(sizeof(MYFILE));
  if (stream==NULL) return NULL;

  stream->d = open(filename,real_mode,0777);
  if (stream->d==-1) {
    free(stream);
    return NULL;
  }

  stream->bufoff = -MYBUFSIZ;
  stream->filoff = 0;
  stream->numchar = MYBUFSIZ;
  stream->flags = myfile_mode;
  stream->status = CLEAN;
  return stream;
}

static int flushbuf(MYFILE *stream) {
  ssize_t l;

  if (stream->status == DIRTY) {
    lseek(stream->d,stream->bufoff,SEEK_SET);
    l = write(stream->d,stream->buf,stream->numchar);
    if (l==-1 || l!=stream->numchar) return -1;
    stream->numchar = 0;
    stream->bufoff += l;
    stream->status = CLEAN;
    return 0;
  }
  stream->numchar = 0;
  stream->bufoff += MYBUFSIZ;
  stream->status = CLEAN;
  return 0;
}

int	myfputc(int c,MYFILE *stream) {
  if (stream==NULL) return -1;
  if (stream->flags & OPEN_WRITE || stream->flags & OPEN_APPEND) {
    if (stream->filoff<stream->bufoff ||
	stream->filoff>=stream->bufoff+MYBUFSIZ) {
      if (flushbuf(stream)==-1) return -1;
    }
    stream->buf[stream->filoff++-stream->bufoff] = (unsigned char)c;
    if (stream->filoff-stream->bufoff>stream->numchar) stream->numchar++;
    stream->status = DIRTY;
    return c;
  }
  return -1;
}

static int fillbuf(MYFILE *stream) {
  ssize_t l;

  l = read(stream->d,stream->buf,MYBUFSIZ);
  if (l==-1) return -1;
  stream->bufoff += MYBUFSIZ;
  stream->numchar = l;
  stream->status = CLEAN;
  return 0;
}

int	myfgetc(MYFILE *stream) {
  int c;
  if (stream==NULL) return -1;
  if (stream->flags & OPEN_READ) {
    if (stream->filoff<stream->bufoff ||
	stream->filoff>=stream->bufoff+stream->numchar) {
      if (stream->numchar<MYBUFSIZ) return -1;
      if (fillbuf(stream)==-1) return -1;
    }
    if (stream->filoff-stream->bufoff>=stream->numchar)
      return -1;
    c = (int)stream->buf[stream->filoff++-stream->bufoff];
    return c;
  }
  return -1;
}

int	myfflush(MYFILE *stream) {
  if (stream==NULL) return -1;
  return 0;
}

int	myfclose(MYFILE *stream) {
  if (stream==NULL) return -1;
  if (stream->flags & OPEN_READ) {
  }
  if (stream->flags & OPEN_WRITE || stream->flags & OPEN_APPEND) {
    flushbuf(stream);
  }
  close(stream->d);
  free(stream);
  return 0;
}
#undef MYSTDIOC
